<#
.SYNOPSIS
   Retrieves startup/shutdown events into a CSV
.DESCRIPTION
   See: http://p0w3rsh3ll.wordpress.com/2013/01/12/get-the-full-reboot-history-for-troubleshooting/
#>

# Store credentials being used for remote computers            
$cred = Get-Credential            
            
# Get the reboot history and store in the $r variable            
# For the local computer
$r =  Get-RebootHistory -Computername 'localhost' -Verbose -Credential $cred

# For multiple computers
# $r =  Get-RebootHistory -Computername 'localhost',"PC2",$env:computername,"PC1","V-PC739" -Verbose -Credential $cred            
            
# Display on console            
$r | Select -Property  ComputerName,TimeCreated,Id,RecordId,Message | ft -AutoSize -Property ComputerName,TimeCreated,Id,Message            
            
# Read the results in Excel            
$r | Select -Property  ComputerName,TimeCreated,Id,RecordId,Message | Out-ExcelReport

Function Out-ExcelReport {
param(
        $Path = "$env:temp\$(Get-Random).csv"
)
    $Input | Export-Csv -Path $Path -Encoding UTF8 -NoTypeInformation -UseCulture
    Invoke-Item -Path $Path
}
 
 
Function Get-EventsHelper {
[CmdletBinding()]
Param(
 $HashTable,
 [string]$Provider,
 [string]$LogName,
 [int32[]]$EventID
)
Begin{
}
Process {
    $SuperHT = @{}
    $SuperHT += @{LogName = $LogName}
    if ($EventID) { $SuperHT += @{ID = $EventID} }
    if ($Provider) { $SuperHT += @{ProviderName = $Provider} }
    try {
        Get-WinEvent -FilterHashtable $SuperHT @HashTable
    } catch {
        Write-Warning -Message "Failed to get events from computer $Computer because $($_.Exception.Message)"
        if ( ($_.Exception.Message -eq "The RPC server is unavailable") -or ($_.CategoryInfo.Reason -eq 'UnauthorizedAccessException')) {
            return
        }
    }
}
End {}
}
 
Function Get-RebootHistory {
 Param(           
    [CmdletBinding()]           
        [parameter(ValueFromPipeline = $True,ValueFromPipeLineByPropertyName = $True)]           
        [Alias('CN','__Server','IPAddress','Server')]           
        [string[]]$Computername = $Env:Computername,           
                     
        [parameter()]           
        [Alias('RunAs')]           
        [System.Management.Automation.Credential()]$Credential = [System.Management.Automation.PSCredential]::Empty                  
    )           
Begin           
{           
    # Make sure we run as admin           
    $usercontext = [Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()           
    $IsAdmin = $usercontext.IsInRole(544)                                                      
    if (-not($IsAdmin))           
    {           
        Write-Warning "Must run powerShell as Administrator to perform these actions"           
        return           
    }            
    $results = @()
}            
Process            
{           
    $ComputerName | ForEach-Object -Process {           
        $allevents = @()
        $Computer = $_           
        # Prepare HT           
        $HT = @{}
        # Supplied Alternate Credentials?                       
        If ($PSBoundParameters['Credential'])  {                       
            $HT.credential = $Credential                       
        }           
        If ($Computer -eq $Env:Computername) {           
            $HT.remove('Credential')           
        } Else {           
            $HT += @{Computername = $Computer}           
        }    
        Write-Verbose -Message "Targeting computer $Computer"      
        $HT += @{ ErrorAction = 'Stop'}
 
        $allevents += Get-EventsHelper -Hashtable $HT -LogName "System" -Provider "User32"
        $allevents += Get-EventsHelper -Hashtable $HT -LogName "System" -Provider "Microsoft-Windows-WER-SystemErrorReporting" -EventID 1001,1018
        $allevents += Get-EventsHelper -Hashtable $HT -LogName "System" -Provider "Microsoft-Windows-Kernel-General" -EventID 1,12,13
        $allevents += Get-EventsHelper -Hashtable $HT -LogName "System" -Provider "Microsoft-Windows-Kernel-Power" -EventID 42,41,109
        $allevents += Get-EventsHelper -Hashtable $HT -LogName "System" -Provider "Microsoft-Windows-Power-Troubleshooter" -EventID 1
        $allevents += Get-EventsHelper -Hashtable $HT -LogName "System" -Provider "Eventlog" -EventID 6005,6006,6008,6013
        $results += ($allevents | ForEach-Object -Process {
                $_ | Add-Member -MemberType NoteProperty -Name ComputerName -Value $Computer
                $_
        } |  Sort-Object -Descending -Property RecordId)
    } # end of Foreach-Object process
    $results           
}
End {}
} # end of function